@@ -1,4 +1,4 @@
-/* $Id: download.c,v 1.3 2011/08/02 20:33:08 imilh Exp $ */
+/* $Id: download.c,v 1.8 2011/08/09 21:58:27 imilh Exp $ */
 
 /*
  * Copyright (c) 2009, 2010 The NetBSD Foundation, Inc.
@@ -32,8 +32,11 @@
 
 #include "pkgin.h"
 
+int fetchTimeout = 15; /* wait 15 seconds before timeout */
+
+/* if db_mtime == NULL, we're downloading a package, pkg_summary otherwise */
 Dlfile *
-download_file(fetchIO *f, char *url, time_t *db_mtime)
+download_file(char *str_url, time_t *db_mtime)
 {
 	/* from pkg_install/files/admin/audit.c */
 	Dlfile			*file;
@@ -43,19 +46,27 @@ download_file(fetchIO *f, char *url, time_t *db_mtime)
 	ssize_t			cur_fetched;
 	time_t			begin_dl, now;
 	struct url_stat	st;
-	int				retry = 3;
+	struct url		*url;
+	fetchIO			*f = NULL;
 
-	if ((fetchStatURL(url, &st, "") < 0) || st.size == -1) {
-		if (db_mtime != NULL)
-			*db_mtime = 0;
+	url = fetchParseURL(str_url);
+
+	if ((f = fetchXGet(url, &st, "")) == NULL)
+		return NULL;
+
+	if (st.size == -1) { /* could not obtain file size */
+		if (db_mtime != NULL) /* we're downloading pkg_summary */
+			*db_mtime = 0; /* ! -1, don't force update */
 
 		return NULL;
 	}
 
 	if (db_mtime != NULL) {
 		if (st.mtime <= *db_mtime) {
-			/* local db is up-to-date */
-			*db_mtime = -1; /* used to identify return type */
+			/* -1 used to identify return type, local summary up-to-date */
+			*db_mtime = -1; 
+
+			fetchIO_close(f);
 
 			return NULL;
 		}
@@ -63,25 +74,11 @@ download_file(fetchIO *f, char *url, time_t *db_mtime)
 		*db_mtime = st.mtime;
 	}
 
-	while (f == NULL) {
-		f = fetchXGetURL(url, &st, "");
-		if (f == NULL) {
-			fprintf(stderr, "%s: fetch failure (%s)%s\n",
-				getprogname(), fetchLastErrString,
-				retry == 1 ? "" : ", retrying");
-			sleep(1);
-			--retry;
-		}
-		if (retry == 0)
-			errx(EXIT_FAILURE, "could not fetch url: %s: %s",
-				url, fetchLastErrString);
-	}
 
-
-	if ((p = strrchr(url, '/')) != NULL)
+	if ((p = strrchr(str_url, '/')) != NULL)
 		p++;
 	else
-		p = (char *)url; /* should not happen */
+		p = (char *)str_url; /* should not happen */
 
 #ifndef _MINIX /* XXX: SSIZE_MAX fails under MINIX */
 	/* st.size is an off_t, it will be > SSIZE_MAX on 32 bits systems */
@@ -124,7 +121,6 @@ download_file(fetchIO *f, char *url, time_t *db_mtime)
 		fflush(stdout);
 	}
 
-	fetchIO_close(f);
 
 	file->buf[buf_len] = '\0';
 	file->size = buf_len;
@@ -134,5 +130,7 @@ download_file(fetchIO *f, char *url, time_t *db_mtime)
 
 	printf("\n");
 
+	fetchIO_close(f);
+
 	return file;
 }
